@@ -86,9 +86,33 @@ class UseCourseGroups(BooleanPreference):
     name = "use_course_groups"
     default = True
     verbose_name = _("Use course groups")
-    help_text = _(
-        "Build or search course groups for every course" " instead of setting classes as groups."
-    )
+    help_text = _("Search course groups for every course instead of setting classes as groups.")
+
+
+@site_preferences_registry.register
+class CreateCourseGroups(BooleanPreference):
+    section = untis_mysql
+    name = "create_course_groups"
+    default = True
+    verbose_name = _("Create non-existing course groups")
+    help_text = _("Only used if 'Use course groups' is enabled.")
+
+
+@site_preferences_registry.register
+class DataCheckNotFoundCourseGroups(BooleanPreference):
+    section = untis_mysql
+    name = "data_check_for_not_found_course_groups"
+    default = True
+    verbose_name = _("Register a data problem if a course group has been not found.")
+
+
+@site_preferences_registry.register
+class CourseGroupsFuzzyMatching(BooleanPreference):
+    section = untis_mysql
+    name = "course_groups_fuzzy_matching"
+    default = False
+    verbose_name = _("Match course groups by a subset of parent groups if no 100% match is found")
+    help_text = _("Works only if 'Use course groups' is activated.")
 
 
 @site_preferences_registry.register
