@@ -13,6 +13,7 @@ from aleksis.core import models as core_models
 from aleksis.core.util.core_helpers import get_site_preferences
 
 from .... import models as mysql_models
+from ....data_checks import CourseGroupNotFoundAndCreated, CourseGroupNotFoundAndNotCreated
 from ..util import (
     TQDM_DEFAULTS,
     compare_m2m,
@@ -120,28 +121,65 @@ def import_lessons(
                 c = classes_ref[class_id]
                 course_classes.append(c)
 
+            course_group_not_found_and_not_created = False
             if get_site_preferences()["untis_mysql__use_course_groups"]:
                 # Negative import_ref denotes a course group
                 group_import_ref = -int("{}{}".format(lesson_id, i))
 
-                # Search by parent groups and subject
+                # Search by parent groups, teachers/owners and subject
                 qs = core_models.Group.objects.filter(
                     parent_groups__in=[c.id for c in course_classes],
                     subject_id=subject.id,
+                    owners__in=[t.id for t in teachers],
                 ).filter(Q(school_term__isnull=True) | Q(school_term=validity_range.school_term))
 
                 # Check if found groups match
-                match = False
+                course_group = None
                 for found_group in qs:
-                    if compare_m2m(course_classes, found_group.parent_groups.all()):
+                    if compare_m2m(course_classes, found_group.parent_groups.all()) and compare_m2m(
+                        teachers, found_group.owners.all()
+                    ):
                         match = True
                         course_group = found_group
                         logger.info(
-                            "    Course group found by searching by parent groups and subject"
+                            "    Course group found by searching by parent groups, "
+                            "teachers (owners) and subject"
                         )
 
+                if (
+                    not match
+                    and get_site_preferences()["untis_mysql__course_groups_fuzzy_matching"]
+                ):
+                    if qs.count() != 1:
+                        logger.warning(
+                            "    Course group not found by searching by parent groups, "
+                            "teachers (owners) and subject (fuzzy matching mode)"
+                        )
+                    else:
+                        for found_group in qs:
+                            if compare_m2m(teachers, found_group.owners.all()):
+                                if match:
+                                    logger.warning(
+                                        "    More than one course group found "
+                                        "by searching by parent groups, "
+                                        "teachers (owners) and subject (fuzzy matching mode)"
+                                    )
+                                    match = False
+                                    course_group = None
+                                else:
+                                    match = True
+                                    course_group = found_group
+                                    logger.info(
+                                        "    Course group found by searching by parent groups, "
+                                        "teachers (owners) and subject (fuzzy matching mode)"
+                                    )
+
                 changed = False
-                if not match:
+                register_data_check = get_site_preferences()[
+                    "untis_mysql__data_check_for_not_found_course_groups"
+                ]
+                if not course_group and get_site_preferences()["untis_mysql__create_course_groups"]:
+
                     # No matching group found
 
                     # Build names and refs for course groups
@@ -174,29 +212,40 @@ def import_lessons(
 
                         changed = True
 
-                # Update owners
-                course_group.owners.set(teachers)
+                    if created and register_data_check:
+                        # No match, new course group created, so create a data problem if activated
+                        CourseGroupNotFoundAndCreated.register_result(course_group)
+                elif register_data_check:
+                    # No match, create deactivated, so create a data problem later if activated
+                    course_group_not_found_and_not_created = True
 
-                # Update import ref
-                if course_group.import_ref_untis != group_import_ref:
-                    course_group.import_ref_untis = group_import_ref
-                    logger.info("    Import reference of course group updated")
-                    changed = True
+                if course_group:
+                    # Update owners
+                    course_group.owners.set(teachers)
 
-                if course_group.subject != subject:
-                    course_group.subject = subject
-                    logger.info("    Subject reference of course group updated")
-                    changed = True
+                    # Update import ref
+                    if course_group.import_ref_untis != group_import_ref:
+                        course_group.import_ref_untis = group_import_ref
+                        logger.info("    Import reference of course group updated")
+                        changed = True
 
-                if course_group.school_term != validity_range.school_term:
-                    course_group.school_term = validity_range.school_term
-                    logger.info("    School term reference of course group updated")
-                    changed = True
+                    if course_group.subject != subject:
+                        course_group.subject = subject
+                        logger.info("    Subject reference of course group updated")
+                        changed = True
 
-                if changed:
-                    course_group.save()
+                    if course_group.school_term != validity_range.school_term:
+                        course_group.school_term = validity_range.school_term
+                        logger.info("    School term reference of course group updated")
+                        changed = True
 
-                groups = [course_group]
+                    if changed:
+                        course_group.save()
+
+                    groups = [course_group]
+
+                else:
+                    groups = []
             else:
                 groups = course_classes
 
@@ -227,8 +276,13 @@ def import_lessons(
                 )
                 logger.info("    New lesson created")
 
+            if course_group_not_found_and_not_created:
+                # Register data problem here because before the lesson object is unknown
+                CourseGroupNotFoundAndNotCreated.register_result(lesson)
+
             # Sync groups
-            lesson.groups.set(groups)
+            if groups:
+                lesson.groups.set(groups)
 
             # Sync teachers
             lesson.teachers.set(teachers)
@@ -236,10 +290,7 @@ def import_lessons(
             # All times for this course
             old_lesson_periods_qs = chronos_models.LessonPeriod.objects.filter(lesson=lesson)
 
-            # If length has changed, delete all lesson periods
-            if old_lesson_periods_qs.count() != len(time_periods):
-                old_lesson_periods_qs.delete()
-                logger.info("    Lesson periods deleted")
+            existing_lesson_period_pks = []
 
             # Sync time periods
             for j, time_period in enumerate(time_periods):
@@ -253,24 +304,29 @@ def import_lessons(
                     room = None
 
                 # Check if an old lesson period is provided
-                old_lesson_period_qs = old_lesson_periods_qs.filter(element_id_untis=j)
+                old_lesson_period_qs = old_lesson_periods_qs.filter(period=time_period)
                 if old_lesson_period_qs.exists():
                     # Update old lesson period
 
-                    old_lesson_period = old_lesson_period_qs[0]
-                    if old_lesson_period.period != time_period or old_lesson_period.room != room:
-                        old_lesson_period.period = time_period
-                        old_lesson_period.room = room
-                        old_lesson_period.save()
-                        logger.info("      Time period and room updated")
+                    lesson_period = old_lesson_period_qs[0]
+                    if lesson_period.room != room or lesson_period.element_id_untis != j:
+                        lesson_period.element_id_untis = j
+                        lesson_period.room = room
+                        lesson_period.save()
+                        logger.info("      Untis reference and room updated")
                 else:
                     # Create new lesson period
 
-                    chronos_models.LessonPeriod.objects.create(
+                    lesson_period = chronos_models.LessonPeriod.objects.create(
                         lesson=lesson, period=time_period, room=room, element_id_untis=j
                     )
-                    logger.info("      New time period added")
+                    logger.info("      New lesson period added")
 
+                existing_lesson_period_pks.append(lesson_period.pk)
+
+            # delete all no-longer existing lesson periods
+            old_lesson_periods_qs.exclude(pk__in=existing_lesson_period_pks).delete()
+            logger.info("    Old lesson periods deleted")
             current_teacher_id = teacher_id
 
     for lesson in chronos_models.Lesson.objects.filter(validity=validity_range):
